<!-- markdownlint-disable MD013 MD043 -->

# OpenSSA: Small Specialist Agents

## Enabling Efficient, Domain-Specific Planning and Reasoning for AI

OpenSSA is an open-source framework for creating efficient, domain-specific AI agents. Build AI assistants for customer support, personalized recommendation engines, or autonomous systems for research. OpenSSA provides the tools to build Small Specialist Agents (SSAs) that solve complex problems in specific domains.

SSAs tackle multi-step problems that require planning and reasoning beyond traditional language models. They apply OODA for deliberative reasoning (OODAR) and iterative, hierarchical task planning (HTP). This "System-2 Intelligence" breaks down complex tasks into manageable steps. SSAs make informed decisions based on domain-specific knowledge. With OpenSSA, create agents that process, generate, and reason about information. This makes them more effective and efficient in solving real-world challenges.

### Key Features

- **Small**: Create lightweight, resource-efficient AI agents through model compression techniques
- **Specialist**: Enhance agent performance with domain-specific facts, rules, heuristics, and fine-tuning for deterministic, accurate results
- **Agents**: Enable goal-oriented, multi-step problem-solving for complex tasks via systematic HTP planning and OODAR reasoning
- **Integration-Ready**: Works seamlessly with popular AI frameworks and tools for easy adoption
- **Extensible Architecture**: Easily integrate new models and domains to expand capabilities
- **Versatile Applications**: Build AI agents for industrial field service, customer support, recommendations, research, and more

### Example Use Cases

#### Boost RAG Performance with Reasoning

OpenSSA significantly boosts the accuracy of Retrieval-Augmented Generation (RAG) systems. It fine-tunes the embedding or completion model with domain-specific knowledge. It also adds the ability to reason about queries and underlying documents. This powerful combination lifts RAG performance by significant margins, overcoming the limitations of generic language models.

#### Enhance Conversational AI for Improved Customer Support

Build AI assistants that provide accurate, context-aware responses in customer support, healthcare, and other domains. OpenSSA's domain-specific fine-tuning capabilities enable you to create AI agents that understand and respond to user queries with unprecedented accuracy and relevance. This leads to improved customer satisfaction, reduced response times, and increased efficiency in handling customer inquiries.

#### Enable Efficient Planning and Reasoning for Problem Solving

OpenSSA enables you to create AI agents that can effectively plan and reason within specific domains to solve complex problems. By leveraging domain-specific knowledge and fine-tuned models, SSAs break down multi-step problems into manageable tasks. They efficiently and precisely solve these tasks, leading to accurate and timely solutions to real-world challenges.

#### Optimize Industrial Field Service Operations

Create AI agents that can guide field service technicians through complex maintenance and repair procedures. By leveraging domain-specific knowledge and reasoning capabilities, SSAs can provide step-by-step instructions, troubleshoot issues, and optimize resource allocation. This results in reduced downtime, increased first-time fix rates, and improved overall efficiency in industrial field service operations.

### Getting Started

1. Install OpenSSA: `pip install openssa` (Python 3.12)
2. Explore the `examples/` directory
3. Start building your own Small Specialist Agents

Detailed tutorials and guides are available in our [Documentation](https://aitomatic.github.io/openssa).

### Contributing

We welcome contributions from the community!

- Join the discussion on our [Community Forum](https://github.com/aitomatic/openssa/discussions)
- Explore the `contrib/` directory for ongoing work and open issues
- Submit pull requests for bug fixes, enhancements, or new features

For more information, see our [Contribution Guide](CONTRIBUTING.md).
